# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AggregatorType(str, Enum):
    """
    The aggregator type.
    """

    """
    allowed enum values
    """
    NONCOMMITTAL = 'NonCommittal'
    MAJORITYVOTE = 'MajorityVote'
    SIMPLEMATCHUP = 'SimpleMatchup'
    LOCATECLUSTER = 'LocateCluster'
    CLASSIFICATION = 'Classification'
    LOCATE = 'Locate'
    BOUNDINGBOX = 'BoundingBox'
    LINE = 'Line'
    TRANSCRIPTION = 'Transcription'
    SINGLEPOINTLOCATE = 'SinglePointLocate'
    FREETEXT = 'FreeText'
    SCRUB = 'Scrub'
    RANKING = 'Ranking'
    MULTIRANKING = 'MultiRanking'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AggregatorType from a JSON string"""
        return cls(json.loads(json_str))


