# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class FilterOperator(str, Enum):
    """
    FilterOperator
    """

    """
    allowed enum values
    """
    EQ = 'Eq'
    NEQ = 'Neq'
    GT = 'Gt'
    GTE = 'Gte'
    LT = 'Lt'
    LTE = 'Lte'
    CONTAINS = 'Contains'
    STARTSWITH = 'StartsWith'
    ENDSWITH = 'EndsWith'
    IN = 'In'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FilterOperator from a JSON string"""
        return cls(json.loads(json_str))


