# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.aggregator_type import AggregatorType
from rapidata.api_client.models.feature_flag import FeatureFlag
from rapidata.api_client.models.i_order_workflow_model import IOrderWorkflowModel
from rapidata.api_client.models.i_referee_model import IRefereeModel
from typing import Optional, Set
from typing_extensions import Self

class CreateJobDefinitionEndpointInput(BaseModel):
    """
    The input for the create job definition endpoint.
    """ # noqa: E501
    definition_name: StrictStr = Field(description="The name of the definition.", alias="definitionName")
    workflow: IOrderWorkflowModel = Field(description="The workflow configuration.")
    referee: IRefereeModel = Field(description="The referee configuration.")
    dataset_id: StrictStr = Field(description="The dataset id.", alias="datasetId")
    feature_flags: Optional[List[FeatureFlag]] = Field(default=None, description="The feature flags.", alias="featureFlags")
    aggregator_type: Optional[AggregatorType] = Field(default=None, alias="aggregatorType")
    __properties: ClassVar[List[str]] = ["definitionName", "workflow", "referee", "datasetId", "featureFlags", "aggregatorType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateJobDefinitionEndpointInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of workflow
        if self.workflow:
            _dict['workflow'] = self.workflow.to_dict()
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if aggregator_type (nullable) is None
        # and model_fields_set contains the field
        if self.aggregator_type is None and "aggregator_type" in self.model_fields_set:
            _dict['aggregatorType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateJobDefinitionEndpointInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "definitionName": obj.get("definitionName"),
            "workflow": IOrderWorkflowModel.from_dict(obj["workflow"]) if obj.get("workflow") is not None else None,
            "referee": IRefereeModel.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "datasetId": obj.get("datasetId"),
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None,
            "aggregatorType": obj.get("aggregatorType")
        })
        return _obj


