# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AgeUserFilterModelAgeGroup(str, Enum):
    """
    AgeUserFilterModelAgeGroup
    """

    """
    allowed enum values
    """
    ENUM_0_MINUS_17 = '0-17'
    ENUM_18_MINUS_29 = '18-29'
    ENUM_30_MINUS_39 = '30-39'
    ENUM_40_MINUS_49 = '40-49'
    ENUM_50_MINUS_64 = '50-64'
    ENUM_65_PLUS = '65+'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgeUserFilterModelAgeGroup from a JSON string"""
        return cls(json.loads(json_str))


